<?php
defined('_JEXEC') or die('Restricted access');

class Delobank extends PaymentRoot
{
    const SESSION_KEY = 'delobank_error_s2g8a9h8w14gb';
    /** @var jshopPaymentMethod */
    public $pm_method;
    private $_verify_request = null;

    public function __construct()
    {
        // Загружаем языковые файлы
        JSFactory::loadExtLanguageFile('delobank');
    }

    /**
     * Вызывается в момент вывода способов оплаты в корзине
     * @param array $params какие-то параметры
     * @param array $settings настройки нашего модуля
     */
    function showPaymentForm($params, $settings)
    {
        return ;
    }

    static function sendToLog($message)
    {
        saveToLog('payment.log', $message);
    }

    /**
     * Метод вызывается при редактировании способа оплаты в админке
     * Он должен выплюнуть форму с настройками
     * @param array|null $params массив со значениями конфига из БД
     */
    function showAdminFormParams($params)
    {
        $arrayParams = $this->settingFields();
        foreach ($arrayParams as $key => $value) {
            if (!isset($params[$key])) {
                $params[$key] = '';
            }
        }
        $orders = JModelLegacy::getInstance('orders', 'JshoppingModel');
        $optionsLang = $this->optionsLang();
        $optionsCurrency = $this->optionsCurrency();
        require __DIR__ . '/views/adminparamsform.php';
    }

    /**
     * Все возможные поля настройки
     * @return array
     */
    public function settingFields()
    {
        return [
            'merch_id' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'pcid',
                    'pcidLength',
                ]
            ],
            'purchase_account_amount_id' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'pcid',
                    'pcidLength',
                ]
            ],
            'back_url_s' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'url',
                ]
            ],
            'back_url_f' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'url',
                ]
            ],
            'url' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'urlDelo',
                ]
            ],
            'lang' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'lang',
                ]
            ],
            'currency' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'int',
                    'currency',
                ]
            ],
            'transaction_end_status' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ]
            ],
            'transaction_failed_status' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ]
            ],
            'info' => [
                'filter' => [
                    'default',
                    // 'htmlspecialchars',
                ],
                'validate' => [
                ]
            ],
            'payment_page' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                ]
            ],
            'check_signature' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                ]
            ],
        ];
    }

    /**
     * Возможные варианты языка
     * @return array
     */
    public function optionsLang()
    {
        return [
            'RU' => 'RU',
            'EN' => 'EN',
        ];
    }

    /**
     * Возможный вариант валюты
     * @return string
     */
    public function optionsCurrency()
    {
        return '643';
    }

    /**
     * Вызывается на последнем шаге корзины
     * @param array $pmconfigs массив с настройками платежного шлюза
     * @param jshopOrder $order объек заказа
     */
    public function showEndForm($pmconfigs, $order)
    {
        $orderId = $order->order_id;
        /** @var jshopCheckoutBuy $checkout */
        $checkout = JSFactory::getModel('checkoutBuy', 'jshop');
        $checkout->setMaxStep(10);
        if (empty($pmconfigs['payment_page'])) {
            // Перебрасываем на стартовую страницу платежного шлюза
            $url = $pmconfigs['url'] . '?' . http_build_query([
                    'merch_id'   => $pmconfigs['merch_id'],
                    'o.order_id' => $orderId,
                    'back_url_s' => $pmconfigs['back_url_s'],
                    'back_url_f' => $pmconfigs['back_url_f'],
                    'lang'       => $pmconfigs['lang'],
                ]);
            $this->redirect($url);
        }
        // Иначе показываем форму, при отправки которой попадаем на стартовую страницу платежного шлюза
        require __DIR__ . '/views/paymentform.php';
    }

    /**
     * Проверка запроса от платежного шлюза
     * Должен вернуть массив следующего типа
     * [
     *      0 => 'код результата',       // int, на основании его выбирается статус заказа
     *      1 => 'сообщение результата', // string, некая формулировка кода результата выше
     *      2 => 'id транзакции',        // string
     *      3 => 'данные транзакции',    // array, массив ключ - значение для данных транзакции
     * ]
     * @param array $pmconfigs настройки нашего модуля
     * @param jshopOrder $order
     * @param string $act типа о каком событии сообщает запрос, у нас всегда будет notify
     * @return array
     */
    public function checkTransaction($pmconfigs, $order, $act)
    {
        $outArray = [99, 'ok', '', []];
        // Если нету основных параметров сразу от ворот поворот
        if (empty($_GET['o_order_id']) || empty($_GET['merch_id']) || empty($_GET['trx_id'])) {
            $error = 'Failed request';
            $this->setErrorInSession($error);
            $outArray[0] = 0;
            $outArray[1] = $error;
            return $outArray;
        }
        $errors = [];
        if (empty($pmconfigs['merch_id'])) {
            $errors[] = 'Empty merch_id';
        } elseif ($pmconfigs['merch_id'] != $_GET['merch_id']) {
            $errors[] = 'Illegal parameter merch_id';
        }
        if (empty($pmconfigs['purchase_account_amount_id'])) {
            $errors[] = 'Empty purchase_account_amount_id';
        }
        if (empty($pmconfigs['transaction_failed_status'])) {
            $errors[] = 'Empty failed_status';
        }
        if (empty($pmconfigs['transaction_end_status'])) {
            $errors[] = 'Empty after_status';
        }
        if ($pmconfigs['currency'] != $this->optionsCurrency()) {
            $errors[] = 'Incorrect currency';
        }
        if (empty($pmconfigs['back_url_f'])) {
            $errors[] = 'Empty back_url_f';
        }
        if (empty($pmconfigs['back_url_s'])) {
            $errors[] = 'Empty back_url_s';
        }
        try {
            $existCertificate = $this->checkCertificate($pmconfigs);
        } catch (Exception $ex) {
            $errors[] = $ex->getMessage();
        }
        if (!$existCertificate) {
            $errors[] = 'Unable to locate certificate';
        }
        if (!empty($errors)) {
            $error = implode("\n", $errors);
            $this->setErrorInSession($error);
            $outArray[0] = 0;
            $outArray[1] = $error;
            return $outArray;
        } else {
            if ($this->isCheckPaymentRequest()) {
                // Первый запрос
                $outArray[0] = 99;
                $outArray[1] = '';
                $outArray[2] = $_GET['trx_id'];
                $outArray[3] = [
                    'order_id'  => $_GET['o_order_id'],
                    'lang_code' => $_GET['lang_code'],
                    'trx_id'    => $_GET['trx_id'],
                    'ts'        => $_GET['ts'],
                ];
                return $outArray;
            } else {
                // Второй запрос
                if ($this->checkSignature($pmconfigs)) {
                    $result = (int)$_GET['result_code'];
                    if ($result != 1) {
                        if (!empty($_GET['ext_result_code'])) {
                            $error = $this->getMessageErrorByCode($_GET['ext_result_code']);
                        } else {
                            $error = 'Unknown payment error';
                        }
                        $this->setErrorInSession($error);
                        $outArray[0] = 3;
                        $outArray[1] = $error;
                        $outArray[2] = $_GET['trx_id'];
                        $outArray[3] = [
                            'order_id'    => $_GET['o_order_id'],
                            'lang_code'   => $_GET['lang_code'],
                            'trx_id'      => $_GET['trx_id'],
                            'ts'          => $_GET['ts'],
                            'result_code' => $_GET['result_code'],
                        ];
                        return $outArray;
                    } else {
                        $outArray[0] = 1;
                        $outArray[1] = '';
                        $outArray[2] = $_GET['trx_id'];
                        $outArray[3] = [
                            'order_id'    => $_GET['o_order_id'],
                            'lang_code'   => $_GET['lang_code'],
                            'trx_id'      => $_GET['trx_id'],
                            'ts'          => $_GET['ts'],
                            'result_code' => $_GET['result_code'],
                        ];
                        return $outArray;
                    }
                } else {
                    $error = 'Digital Signature Verification Failed';
                    $this->setErrorInSession($error);
                    $outArray[0] = 0;
                    $outArray[1] = $error;
                    return $outArray;
                }
            }
        }
        return $outArray;
    }

    /**
     * @param array $pmconfigs настройки нашего модуля
     * @param jshopOrder $order
     * @param int $rescode код результата
     *      0 - ошибка, статус не меняется
     *      99 - первый запрос прошел успешно, статус не меняется
     *      3 - ошибка на втором запросе, которая приводить к отмене заказа, тут может быть отмена платежа, нет денег и т.д., статус неуспешная оплата
     *      1 - второй запрос успешно, статус оплачено
     * @throws Exception
     */
    public function nofityFinish($pmconfigs, $order, $rescode)
    {
        if ($rescode == 0) {
            $error = $this->getErrorFromSession();
            if ($this->isCheckPaymentRequest()) {
                $xmlResponse = $this->createFailedResponse($error, 'check');
            } else {
                $xmlResponse = $this->createFailedResponse($error, 'register');
            }
        } elseif ($rescode == 99) {
            $xmlResponse = $this->paymentAvailabilityPositiveResponse($order, $pmconfigs);
        } elseif ($rescode == 3) {
            // При отмене платежа, отвечаем тоже положительно, но с формулировкой причины вместо 'ок'
            $error = $this->getErrorFromSession();
            $xmlResponse = $this->registerResponseSuccessful($error);
        } else {
            // Еще последняя проверка поменялся ли статус у заказа
            if ($order->order_created == 1) {
                $xmlResponse = $this->registerResponseSuccessful('ok');
            } else {
                $xmlResponse = $this->createFailedResponse('Not change status order', 'register');
            }
        }
        $this->sendResponse($xmlResponse);
    }

    /**
     * Делает 303 редирект
     * @param string $url
     */
    private function redirect($url)
    {
        header('HTTP/1.1 303 See Other');
        header('Location: ' . $url);
        exit();
    }

    /**
     * Метод разбирает параметры из первого запроса от платежного шлюза
     * Должен вернуть массив с минимум следующими параметрами, они нужны CMS для дальнейшей обработки
     * [
     *      'order_id'          => 1,                                  // id заказа
     *      'hash'              => 'F1G4E3MUIY4353533GWFRWFSDGR3T3T3', // некий хэш
     *      'checkHash'         => 1,                                  // нужно ли проверять этот хэш
     *      'checkReturnParams' => 1                                   // нужно ли проверять входные параметры
     * ];
     * @param array $pmconfigs настройки нашего модуля способа оплаты
     * @return array
     * @throws Exception
     */
    public function getUrlParams($pmconfigs)
    {
        $urlParams = [
            'order_id'          => 0,
            'hash'              => '',
            'checkHash'         => 0,
            'checkReturnParams' => 1,
        ];
        // Нам надо только получить id заказа из нашего параметра и указать необходимость проверки входных параметров
        $orderId = JFactory::getApplication()->input->getInt('o_order_id', null);
        $urlParams['order_id'] = $orderId;
        return $urlParams;
    }

    /**
     * @param string $errorMessage
     * @param string $request ответ на какой запрос, может быть check|register
     * @return string
     */
    private function createFailedResponse($errorMessage, $request = 'check')
    {
        if ('register' === $request) {
            return '<register-payment-response>' . $this->xmlResult(2, $errorMessage) . '</register-payment-response>';
        } else {
            return '<payment-avail-response>' . $this->xmlResult(2, $errorMessage) . '</payment-avail-response>';
        }
    }

    /**
     * Возвращает кусок xml с <result>
     * @param int $code 1 – Успех. Продолжить оплату. 2 – Неудача. Прервать процесс оплаты
     * @param string $desc описание ошибки или успеха
     * @return string xml
     */
    private function xmlResult($code, $desc)
    {
        return '<result><code>' . $code . '</code><desc><![CDATA[' . $desc . ']]></desc></result>';
    }

    /**
     * Запоминает ошибку в сессии, чтобы ее в дальнейшем получить
     * @param string $error
     */
    private function setErrorInSession($error)
    {
        $_SESSION[self::SESSION_KEY] = $error;
    }

    /**
     * Возвращает ошибку из сессии
     * @return string
     */
    private function getErrorFromSession()
    {
        if (!empty($_SESSION[self::SESSION_KEY])) {
            $error = $_SESSION[self::SESSION_KEY];
            unset($_SESSION[self::SESSION_KEY]);
            return $error;
        }
        return '';
    }

    /**
     * Определяет по параметрам запроса, запрос ли это на проверку платежа
     * @return bool
     */
    private function isCheckPaymentRequest()
    {
        if (!isset($_GET['result_code'])) {
            return true;
        }
        return false;
    }

    /**
     * Выставляет правильные заголовки и выплевывает ответ
     * @param string $xml xml ответа
     */
    private function sendResponse($xml)
    {
        header('Content-type: application/xml');
        exit($xml);
    }

    /**
     * Проверка существования сертификата эквайера. Может вызываться на обоих фазах проведения платежа.
     * Убедимся, что пользователь залил серификат по фтп на сайт и корректно прописал путь до него в настройках платежного модуля
     * @param array $paymentSettings
     * @throws Exception
     * @return bool
     */
    private function checkCertificate($paymentSettings)
    {
        if (empty($paymentSettings['check_signature'])) {
            // Если проверка ЭЦП выключена
            return true;
        }
        $certFile = $this->getPathCert();
        if (!file_exists($certFile) || !is_file($certFile)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Положительный ответ на проверку возможности проведения платежа
     * @param jshopOrder $order
     * @param array $paymentSettings настройки
     * @return string
     */
    private function paymentAvailabilityPositiveResponse($order, $paymentSettings)
    {
        $descShortPurchase = 'Оплата заказа № ' . $order->order_id;
        $descLongPurchase = 'Оплата заказа № ' . $order->order_id;
        $amount = $this->rubToPennies($order->order_total);
        return '<payment-avail-response>' . $this->xmlResult(1, 'ok')
            . '<purchase>'
            . '<shortDesc><![CDATA[' . $descShortPurchase . ']]></shortDesc>'
            . '<longDesc><![CDATA[' . $descLongPurchase . ']]></longDesc>'
            . '<account-amount>'
            . '<amount>' . $amount . '</amount>'
            . '<currency>' . $paymentSettings['currency'] . '</currency>'
            . '<exponent>2</exponent>'
            . '</account-amount>'
            . '</purchase>'
            . '</payment-avail-response>';
    }

    /**
     * Преобразуем рубли в копейки
     * @param string $amount
     * @return int
     */
    private function rubToPennies($amount)
    {
        $amount = number_format($amount, 2, '', '');
        return (int)$amount;
    }

    /**
     * Преобразуем копейки в рубли
     * @param string $amount
     * @return string
     */
    private function penniesToRub($amount)
    {
        return number_format($amount / 100, 2, '.', '');
    }

    /**
     * Возвращает положительный ответ на запрос фиксации платежа от платежного шлюза
     * @param string $desc описание успешной регистрации платежа
     * @return string xml
     */
    private function registerResponseSuccessful($desc = 'ok')
    {
        return '<register-payment-response>' . $this->xmlResult(1, $desc) . '</register-payment-response>';
    }

    /**
     * Проверка подписи (ЭЦП) в запросе от эквайера
     * @param array $paymentSettings настройки нашего модуля
     * @return bool true - если проверка пройдена и false если не пройдена по какой-либо причине
     */
    private function checkSignature($paymentSettings)
    {
        if (empty($paymentSettings['check_signature'])) {
            // Если проверка ЭЦП выключена
            return true;
        }
        $signature = base64_decode($_GET['signature']);
        $data = $this->getDataFromUrl();
        $certFile = $this->getPathCert();
        // Получаем из сертификата публичный ключ для проверки
        $publicKey = openssl_pkey_get_public('file://' . $certFile);
        if ($publicKey === false) {
            return false;
        }
        // Проверяем подпись и освобождаем ключ
        $signatureCheck = openssl_verify($data, $signature, $publicKey, OPENSSL_ALGO_SHA1);
        openssl_free_key($publicKey);
        if ($signatureCheck !== 1) {
            return false;
        }
        return true;
    }

    /**
     * Выделяет подписанные данные из запроса
     * @return string часть url-а c доменом до &signature=
     */
    private function getDataFromUrl()
    {
        $url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? 'https' : 'http') . "://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
        $pos = strpos($url, '&signature=');
        return substr($url, 0, $pos);
    }

    /**
     * Возвращает путь до файла с сертификатом
     * @return string
     */
    private function getPathCert()
    {
        if (isset($_SERVER['DELOBANK_ENVIRONMENT'])) {
            $certFile = JPATH_COMPONENT . '/payments/delobank/cert/delobank_test.crt';
        } else {
            $certFile = JPATH_COMPONENT . '/payments/delobank/cert/delobank_production.crt';
        }
        return $certFile;
    }

    /**
     * Возвращает сообщение ошибки по ее коду
     * @param string $code
     * @return string
     */
    private function getMessageErrorByCode($code)
    {
        $errors = [
            '-2' => _JSHOP_DELOBANK_ERROR_REGISTER__2,
            '-3' => _JSHOP_DELOBANK_ERROR_REGISTER__3,
            '-4' => _JSHOP_DELOBANK_ERROR_REGISTER__4,
            '2'  => _JSHOP_DELOBANK_ERROR_REGISTER_2,
            '4'  => _JSHOP_DELOBANK_ERROR_REGISTER_4,
            '21' => _JSHOP_DELOBANK_ERROR_REGISTER_21,
            '53' => _JSHOP_DELOBANK_ERROR_REGISTER_53,
        ];
        if (isset($errors[$code])) {
            return $errors[$code];
        }
        return 'Ошибка проведения платежа с кодом ошибки: ' . $code;
    }
}
