<?php
namespace Delo;

require_once __DIR__ . '/components/com_jshopping/payments/delobank/delobank.php';

use Joomla\CMS\Factory;
use \Delobank;

/**
 * Класс установщик нашего способа оплаты
 * @package Delo
 */
class Update
{
    const NAME = 'delobank';
    /** @var Delobank */
    private $model;

    public function __construct()
    {
        $this->model = new Delobank();
    }

    public function run()
    {
        $paymentId = $this->existPaymentMethod();
        if (!empty($paymentId)) {
            $this->updatePaymentMethod($paymentId);
        } else {
            $this->addPaymentMethod();
        }
    }

    /**
     * Обновляет настройки способа оплаты
     * @param int $paymentId
     */
    private function updatePaymentMethod($paymentId)
    {
        $existSettings = $this->getExistPaymentParam($paymentId);
        $defaultSettings = $this->defaultSettingsValues();
        // Если настройка исчезает, удаляем ее
        $newExistSettings = [];
        foreach ($existSettings as $key => $val) {
            if (isset($defaultSettings[$key])) {
                $newExistSettings[$key] = $val;
            }
        }
        // Если настройки нет или она пустая выставляем ее в default, иначе оставляем существующее значение
        foreach ($defaultSettings as $key => $val) {
            if (empty($newExistSettings[$key])) {
                $newExistSettings[$key] = $val;
            }
        }
        // Получили обновленный массив настроек, обновим БД
        $db = Factory::getDBO();
        $paymentParams = $this->formateParamForDB($newExistSettings);
        $paymentMethod = new \stdClass();
        $paymentMethod->payment_id = $paymentId;
        $paymentMethod->payment_params = $paymentParams;
        $db->updateObject('#__jshopping_payment_method', $paymentMethod, 'payment_id');
    }

    /**
     * Добавляем в БД информацию о нашем способе оплаты,
     * чтобы в дальнейшем она отображалась взаимодействовала с joomshopping, т.е. показывалась в админке, корзине
     */
    private function addPaymentMethod()
    {
        $db = Factory::getDBO();
        $paymentMethod = $this->createPaymentMethod();
        $db->insertObject('#__jshopping_payment_method', $paymentMethod, 'payment_id');
    }

    /**
     * Возвращает объект с данными по методу оплаты
     * @return \stdClass
     */
    private function createPaymentMethod()
    {
        $paymentMethod = new \stdClass();
        $paymentMethod->payment_code = self::NAME;
        $paymentMethod->payment_class = self::NAME;
        $paymentMethod->scriptname = self::NAME;
        $paymentMethod->payment_publish = 0;
        $paymentMethod->payment_ordering = $this->getSort();
        $paymentMethod->payment_type = 2;
        $paymentMethod->price = 0.00;
        $paymentMethod->price_type = 0;
        $paymentMethod->tax_id = 1;
        $paymentMethod->image = '';
        $paymentMethod->show_descr_in_email = 1;
        $paymentMethod->order_description = '';
        // Выставим настройки по умолчанию
        $paymentMethod->payment_params = $this->formateParamForDB($this->defaultSettingsValues());
        $paymentMethod->{'name_en-GB'} = _JSHOP_DELOBANK_TITLE_DEFAULT;
        $paymentMethod->{'description_en-GB'} = _JSHOP_DELOBANK_TERMS_DEFAULT;
        $paymentMethod->{'name_de-DE'} = _JSHOP_DELOBANK_TITLE_DEFAULT;
        $paymentMethod->{'description_de-DE'} = _JSHOP_DELOBANK_TERMS_DEFAULT;
        $paymentMethod->{'name_ru-RU'} = _JSHOP_DELOBANK_TITLE_DEFAULT;
        $paymentMethod->{'description_ru-RU'} = _JSHOP_DELOBANK_TERMS_DEFAULT;
        return $paymentMethod;
    }

    /**
     * Defaults значения настроек
     * @return array
     */
    private function defaultSettingsValues()
    {
        $defaultSettings = $this->getDefaultParamValues();
        // Дополняем те у котрых нет default значении, пустыми значениями, чтобы в БД заранее были записи для каждой настройки
        $fields = $this->model->settingFields();
        foreach ($fields as $fieldName => $fieldSetting) {
            if (!isset($defaultSettings[$fieldName])) {
                $defaultSettings[$fieldName] = '';
            }
        }
        return $defaultSettings;
    }

    /**
     * Значения по умолчанию для настроек нашего метода оплаты
     * @return array
     */
    private function getDefaultParamValues()
    {
        return [
            'purchase_account_amount_id' => (isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 'sdaf324fwqfasf' : ''),
            'merch_id'                   => (isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 'sdaf324fwqfasf' : ''),
            'back_url_s'      => 'https://' . $_SERVER['HTTP_HOST'] . '/index.php?option=com_jshopping&controller=checkout&task=finish',
            'back_url_f'      => 'https://' . $_SERVER['HTTP_HOST'] . '/index.php?option=com_jshopping&controller=checkout&task=step5',
            'url'             => (isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 'https://testmpi.myskb.ru/payment/start.wsm' : 'https://mpi.skbbank.ru/payment/start.wsm'),
            'lang'            => 'RU',
            'currency'        => $this->model->optionsCurrency(),
            'info'            => _JSHOP_DELOBANK_INFO_DEFAULT,
            'payment_page'    => 0,
            'check_signature' => 1,
        ];
    }

    /**
     * Возвращает следующий номер сортировки для размещения нашего способа оплаты
     * @return int
     */
    private function getSort()
    {
        return 0;
    }

    /**
     * Возврвщает id нашего метода оплаты если он уже есть в БД
     * @return int
     */
    private function existPaymentMethod()
    {
        $db = Factory::getDBO();
        $q = 'SELECT payment_id FROM #__jshopping_payment_method WHERE payment_code = "' . self::NAME . '"';
        $r = $db->setQuery($q)->execute();
        $arr = $r->fetch();
        if (!empty($arr['payment_id'])) {
            return (int)$arr['payment_id'];
        }
        return 0;
    }

    /**
     * @param array $settings массив настроек
     * @return string
     */
    private function formateParamForDB($settings)
    {
        $params = '';
        foreach ($settings as $key => $value) {
            $params .= $key . '=' . $value . "\n";
        }
        return $params;
    }

    /**
     * Достаем существующие в БД настройки способа оплаты
     * @param int $paymentId
     * @return array
     */
    private function getExistPaymentParam($paymentId)
    {
        $db = Factory::getDBO();
        $q = 'SELECT payment_params FROM #__jshopping_payment_method WHERE payment_id = ' . $paymentId;
        $r = $db->setQuery($q)->execute();
        $arr = $r->fetch();
        $existSettings = [];
        if (!empty($arr['payment_params'])) {
            $params = trim($arr['payment_params']);
            $params = explode("\n", $params);
            if (!empty($params)) {
                foreach ($params as $param) {
                    list ($key, $val) = explode('=', $param);
                    $existSettings[$key] = $val;
                }
            }
        }
        return $existSettings;
    }
}

(new Update())->run();
